@@ -32,12 +32,14 @@ import (
 	"github.com/google/go-containerregistry/pkg/name"
 	v1 "github.com/google/go-containerregistry/pkg/v1"
 	"github.com/google/go-containerregistry/pkg/v1/remote"
-
+	intotov1 "github.com/in-toto/attestation/go/v1"
 	"github.com/sigstore/cosign/v2/cmd/cosign/cli/fulcio"
 	"github.com/sigstore/cosign/v2/cmd/cosign/cli/fulcio/fulcioverifier"
 	"github.com/sigstore/cosign/v2/cmd/cosign/cli/options"
 	"github.com/sigstore/cosign/v2/cmd/cosign/cli/rekor"
 	"github.com/sigstore/cosign/v2/cmd/cosign/cli/sign/privacy"
+	"github.com/sigstore/cosign/v2/internal/auth"
+	"github.com/sigstore/cosign/v2/internal/key"
 	icos "github.com/sigstore/cosign/v2/internal/pkg/cosign"
 	ifulcio "github.com/sigstore/cosign/v2/internal/pkg/cosign/fulcio"
 	ipayload "github.com/sigstore/cosign/v2/internal/pkg/cosign/payload"
@@ -46,6 +48,7 @@ import (
 	"github.com/sigstore/cosign/v2/internal/pkg/cosign/tsa/client"
 	"github.com/sigstore/cosign/v2/internal/ui"
 	"github.com/sigstore/cosign/v2/pkg/cosign"
+	cbundle "github.com/sigstore/cosign/v2/pkg/cosign/bundle"
 	"github.com/sigstore/cosign/v2/pkg/cosign/pivkey"
 	"github.com/sigstore/cosign/v2/pkg/cosign/pkcs11key"
 	cremote "github.com/sigstore/cosign/v2/pkg/cosign/remote"
@@ -54,10 +57,15 @@ import (
 	ociremote "github.com/sigstore/cosign/v2/pkg/oci/remote"
 	"github.com/sigstore/cosign/v2/pkg/oci/walk"
 	sigs "github.com/sigstore/cosign/v2/pkg/signature"
+	"github.com/sigstore/cosign/v2/pkg/types"
+	"github.com/sigstore/rekor/pkg/generated/models"
+	"github.com/sigstore/sigstore-go/pkg/sign"
 	"github.com/sigstore/sigstore/pkg/cryptoutils"
 	"github.com/sigstore/sigstore/pkg/signature"
+	"github.com/sigstore/sigstore/pkg/signature/dsse"
 	signatureoptions "github.com/sigstore/sigstore/pkg/signature/options"
 	sigPayload "github.com/sigstore/sigstore/pkg/signature/payload"
+	"google.golang.org/protobuf/encoding/protojson"
 
 	// Loads OIDC providers
 	_ "github.com/sigstore/cosign/v2/pkg/providers/all"
@@ -123,8 +131,7 @@ func ParseOCIReference(ctx context.Context, refStr string, opts ...name.Option)
 		return nil, fmt.Errorf("parsing reference: %w", err)
 	}
 	if _, ok := ref.(name.Digest); !ok {
-		msg := fmt.Sprintf(ui.TagReferenceMessage, refStr)
-		ui.Warnf(ctx, msg)
+		ui.Warnf(ctx, ui.TagReferenceMessage, refStr)
 	}
 	return ref, nil
 }
@@ -138,14 +145,8 @@ func SignCmd(ro *options.RootOptions, ko options.KeyOpts, signOpts options.SignO
 	ctx, cancel := context.WithTimeout(context.Background(), ro.Timeout)
 	defer cancel()
 
-	sv, err := SignerFromKeyOpts(ctx, signOpts.Cert, signOpts.CertChain, ko)
-	if err != nil {
-		return fmt.Errorf("getting signer: %w", err)
-	}
-	defer sv.Close()
-	dd := cremote.NewDupeDetector(sv)
-
 	var staticPayload []byte
+	var err error
 	if signOpts.PayloadPath != "" {
 		ui.Infof(ctx, "Using payload from: %s", signOpts.PayloadPath)
 		staticPayload, err = os.ReadFile(filepath.Clean(signOpts.PayloadPath))
@@ -186,7 +187,11 @@ func SignCmd(ro *options.RootOptions, ko options.KeyOpts, signOpts options.SignO
 			} else if err != nil {
 				return fmt.Errorf("accessing image: %w", err)
 			}
-			err = signDigest(ctx, digest, staticPayload, ko, signOpts, annotations, dd, sv, se)
+			if signOpts.NewBundleFormat {
+				err = signDigestBundle(ctx, digest, ko, signOpts)
+			} else {
+				err = signDigest(ctx, digest, staticPayload, ko, signOpts, annotations, se)
+			}
 			if err != nil {
 				return fmt.Errorf("signing digest: %w", err)
 			}
@@ -205,7 +210,11 @@ func SignCmd(ro *options.RootOptions, ko options.KeyOpts, signOpts options.SignO
 				return fmt.Errorf("computing digest: %w", err)
 			}
 			digest := ref.Context().Digest(d.String())
-			err = signDigest(ctx, digest, staticPayload, ko, signOpts, annotations, dd, sv, se)
+			if signOpts.NewBundleFormat {
+				err = signDigestBundle(ctx, digest, ko, signOpts)
+			} else {
+				err = signDigest(ctx, digest, staticPayload, ko, signOpts, annotations, se)
+			}
 			if err != nil {
 				return fmt.Errorf("signing digest: %w", err)
 			}
@@ -218,9 +227,170 @@ func SignCmd(ro *options.RootOptions, ko options.KeyOpts, signOpts options.SignO
 	return nil
 }
 
+func signDigestBundle(ctx context.Context, digest name.Digest, ko options.KeyOpts, signOpts options.SignOptions) error {
+	digestParts := strings.Split(digest.DigestStr(), ":")
+	if len(digestParts) != 2 {
+		return fmt.Errorf("unable to parse digest %s", digest.DigestStr())
+	}
+
+	subject := intotov1.ResourceDescriptor{
+		Digest: map[string]string{digestParts[0]: digestParts[1]},
+	}
+
+	statement := &intotov1.Statement{
+		Type:          intotov1.StatementTypeUri,
+		Subject:       []*intotov1.ResourceDescriptor{&subject},
+		PredicateType: types.CosignSignPredicateType,
+	}
+
+	payload, err := protojson.Marshal(statement)
+	if err != nil {
+		return err
+	}
+
+	if ko.SigningConfig != nil {
+		var keypair sign.Keypair
+		var ephemeralKeypair bool
+		var idToken string
+		var sv *SignerVerifier
+		var err error
+
+		if ko.Sk || ko.Slot != "" || ko.KeyRef != "" || signOpts.Cert != "" {
+			sv, _, err = SignerFromKeyOpts(ctx, signOpts.Cert, signOpts.CertChain, ko)
+			if err != nil {
+				return fmt.Errorf("getting signer: %w", err)
+			}
+			keypair, err = key.NewSignerVerifierKeypair(sv, ko.DefaultLoadOptions)
+			if err != nil {
+				return fmt.Errorf("creating signerverifier keypair: %w", err)
+			}
+		} else {
+			keypair, err = sign.NewEphemeralKeypair(nil)
+			if err != nil {
+				return fmt.Errorf("generating keypair: %w", err)
+			}
+			ephemeralKeypair = true
+		}
+		defer func() {
+			if sv != nil {
+				sv.Close()
+			}
+		}()
+
+		if ephemeralKeypair || ko.IssueCertificateForExistingKey {
+			idToken, err = auth.RetrieveIDToken(ctx, auth.IDTokenConfig{
+				TokenOrPath:      ko.IDToken,
+				DisableProviders: ko.OIDCDisableProviders,
+				Provider:         ko.OIDCProvider,
+				AuthFlow:         ko.FulcioAuthFlow,
+				SkipConfirm:      ko.SkipConfirmation,
+				OIDCServices:     ko.SigningConfig.OIDCProviderURLs(),
+				ClientID:         ko.OIDCClientID,
+				ClientSecret:     ko.OIDCClientSecret,
+				RedirectURL:      ko.OIDCRedirectURL,
+			})
+			if err != nil {
+				return fmt.Errorf("retrieving ID token: %w", err)
+			}
+		}
+
+		content := &sign.DSSEData{
+			Data:        payload,
+			PayloadType: "application/vnd.in-toto+json",
+		}
+		bundle, err := cbundle.SignData(ctx, content, keypair, idToken, ko.SigningConfig, ko.TrustedMaterial)
+		if err != nil {
+			return fmt.Errorf("signing bundle: %w", err)
+		}
+
+		regOpts := signOpts.Registry
+		ociremoteOpts, err := regOpts.ClientOpts(ctx)
+		if err != nil {
+			return fmt.Errorf("constructing client options: %w", err)
+		}
+		return ociremote.WriteAttestationNewBundleFormat(digest, bundle, types.CosignSignPredicateType, ociremoteOpts...)
+	}
+
+	sv, genKey, err := SignerFromKeyOpts(ctx, signOpts.Cert, signOpts.CertChain, ko)
+	if err != nil {
+		return fmt.Errorf("getting signer: %w", err)
+	}
+	if genKey || ko.IssueCertificateForExistingKey {
+		sv, err = KeylessSigner(ctx, ko, sv)
+		if err != nil {
+			return fmt.Errorf("getting Fulcio signer: %w", err)
+		}
+	}
+	defer sv.Close()
+
+	wrapped := dsse.WrapSigner(sv, types.IntotoPayloadType)
+	signedPayload, err := wrapped.SignMessage(bytes.NewReader(payload), signatureoptions.WithContext(ctx))
+	if err != nil {
+		return fmt.Errorf("signing: %w", err)
+	}
+
+	var timestampBytes []byte
+	if ko.TSAServerURL != "" {
+		tsaPayload, err := cosign.GetDSSESigBytes(signedPayload)
+		if err != nil {
+			return err
+		}
+		tc := client.NewTSAClient(ko.TSAServerURL)
+		if ko.TSAClientCert != "" {
+			tc = client.NewTSAClientMTLS(ko.TSAServerURL,
+				ko.TSAClientCACert,
+				ko.TSAClientCert,
+				ko.TSAClientKey,
+				ko.TSAServerName,
+			)
+		}
+		timestampBytes, err = tsa.GetTimestampedSignature(tsaPayload, tc)
+		if err != nil {
+			return err
+		}
+	}
+
+	signerBytes, err := sv.Bytes(ctx)
+	if err != nil {
+		return err
+	}
+
+	var rekorEntry *models.LogEntryAnon
+	shouldUpload, err := ShouldUploadToTlog(ctx, ko, digest, signOpts.TlogUpload)
+	if err != nil {
+		return fmt.Errorf("should upload to tlog: %w", err)
+	}
+	if shouldUpload {
+		rClient, err := rekor.NewClient(ko.RekorURL)
+		if err != nil {
+			return err
+		}
+		rekorEntry, err = cosign.TLogUploadDSSEEnvelope(ctx, rClient, signedPayload, signerBytes)
+		if err != nil {
+			return err
+		}
+	}
+
+	regOpts := signOpts.Registry
+	ociremoteOpts, err := regOpts.ClientOpts(ctx)
+	if err != nil {
+		return fmt.Errorf("constructing client options: %w", err)
+	}
+
+	pubKey, err := sv.PublicKey()
+	if err != nil {
+		return err
+	}
+
+	bundleBytes, err := cbundle.MakeNewBundle(pubKey, rekorEntry, payload, signedPayload, signerBytes, timestampBytes)
+	if err != nil {
+		return err
+	}
+	return ociremote.WriteAttestationNewBundleFormat(digest, bundleBytes, types.CosignSignPredicateType, ociremoteOpts...)
+}
+
 func signDigest(ctx context.Context, digest name.Digest, payload []byte, ko options.KeyOpts, signOpts options.SignOptions,
-	annotations map[string]interface{},
-	dd mutate.DupeDetector, sv *SignerVerifier, se oci.SignedEntity) error {
+	annotations map[string]interface{}, se oci.SignedEntity) error {
 	var err error
 	// The payload can be passed to skip generation.
 	if len(payload) == 0 {
@@ -234,6 +404,19 @@ func signDigest(ctx context.Context, digest name.Digest, payload []byte, ko opti
 		}
 	}
 
+	sv, genKey, err := SignerFromKeyOpts(ctx, signOpts.Cert, signOpts.CertChain, ko)
+	if err != nil {
+		return fmt.Errorf("getting signer: %w", err)
+	}
+	if genKey || ko.IssueCertificateForExistingKey {
+		sv, err = KeylessSigner(ctx, ko, sv)
+		if err != nil {
+			return fmt.Errorf("getting Fulcio signer: %w", err)
+		}
+	}
+	defer sv.Close()
+	dd := cremote.NewDupeDetector(sv)
+
 	var s icos.Signer
 	s = ipayload.NewSigner(sv)
 	if sv.Cert != nil {
@@ -391,8 +574,8 @@ func signerFromSecurityKey(ctx context.Context, keySlot string) (*SignerVerifier
 	}, nil
 }
 
-func signerFromKeyRef(ctx context.Context, certPath, certChainPath, keyRef string, passFunc cosign.PassFunc) (*SignerVerifier, error) {
-	k, err := sigs.SignerVerifierFromKeyRef(ctx, keyRef, passFunc)
+func signerFromKeyRef(ctx context.Context, certPath, certChainPath, keyRef string, passFunc cosign.PassFunc, defaultLoadOptions *[]signature.LoadOption) (*SignerVerifier, error) {
+	k, err := sigs.SignerVerifierFromKeyRef(ctx, keyRef, passFunc, defaultLoadOptions)
 	if err != nil {
 		return nil, fmt.Errorf("reading key: %w", err)
 	}
@@ -499,23 +682,6 @@ func signerFromKeyRef(ctx context.Context, certPath, certChainPath, keyRef strin
 	if _, err := cosign.TrustedCert(leafCert, rootPool, subPool); err != nil {
 		return nil, fmt.Errorf("unable to validate certificate chain: %w", err)
 	}
-	// Verify SCT if present in the leaf certificate.
-	contains, err := cosign.ContainsSCT(leafCert.Raw)
-	if err != nil {
-		return nil, err
-	}
-	if contains {
-		pubKeys, err := cosign.GetCTLogPubs(ctx)
-		if err != nil {
-			return nil, fmt.Errorf("getting CTLog public keys: %w", err)
-		}
-		var chain []*x509.Certificate
-		chain = append(chain, leafCert)
-		chain = append(chain, certChain...)
-		if err := cosign.VerifyEmbeddedSCT(context.Background(), chain, pubKeys); err != nil {
-			return nil, err
-		}
-	}
 	certSigner.Chain = certChainBytes
 
 	return certSigner, nil
@@ -536,12 +702,16 @@ func signerFromNewKey() (*SignerVerifier, error) {
 	}, nil
 }
 
-func keylessSigner(ctx context.Context, ko options.KeyOpts, sv *SignerVerifier) (*SignerVerifier, error) {
+func KeylessSigner(ctx context.Context, ko options.KeyOpts, sv *SignerVerifier) (*SignerVerifier, error) {
 	var (
 		k   *fulcio.Signer
 		err error
 	)
 
+	if _, ok := sv.SignerVerifier.(*signature.ED25519phSignerVerifier); ok {
+		return nil, fmt.Errorf("ed25519ph unsupported by Fulcio")
+	}
+
 	if ko.InsecureSkipFulcioVerify {
 		if k, err = fulcio.NewSigner(ctx, ko, sv); err != nil {
 			return nil, fmt.Errorf("getting key from Fulcio: %w", err)
@@ -559,7 +729,7 @@ func keylessSigner(ctx context.Context, ko options.KeyOpts, sv *SignerVerifier)
 	}, nil
 }
 
-func SignerFromKeyOpts(ctx context.Context, certPath string, certChainPath string, ko options.KeyOpts) (*SignerVerifier, error) {
+func SignerFromKeyOpts(ctx context.Context, certPath string, certChainPath string, ko options.KeyOpts) (*SignerVerifier, bool, error) {
 	var sv *SignerVerifier
 	var err error
 	genKey := false
@@ -567,21 +737,16 @@ func SignerFromKeyOpts(ctx context.Context, certPath string, certChainPath strin
 	case ko.Sk:
 		sv, err = signerFromSecurityKey(ctx, ko.Slot)
 	case ko.KeyRef != "":
-		sv, err = signerFromKeyRef(ctx, certPath, certChainPath, ko.KeyRef, ko.PassFunc)
+		sv, err = signerFromKeyRef(ctx, certPath, certChainPath, ko.KeyRef, ko.PassFunc, ko.DefaultLoadOptions)
 	default:
 		genKey = true
 		ui.Infof(ctx, "Generating ephemeral keys...")
 		sv, err = signerFromNewKey()
 	}
 	if err != nil {
-		return nil, err
+		return nil, false, err
 	}
-
-	if ko.IssueCertificateForExistingKey || genKey {
-		return keylessSigner(ctx, ko, sv)
-	}
-
-	return sv, nil
+	return sv, genKey, nil
 }
 
 type SignerVerifier struct {
@@ -624,9 +789,6 @@ func fetchLocalSignedPayload(sig oci.Signature) (*cosign.LocalSignedPayload, err
 	}
 	if sigCert != nil {
 		signedPayload.Cert = base64.StdEncoding.EncodeToString(sigCert.Raw)
-		if err != nil {
-			return nil, err
-		}
 	} else {
 		signedPayload.Cert = ""
 	}
